import React, { useEffect, useState, useRef } from 'react';
import { isFormEvent, canClickNode, isShaken } from '@gant-lowcode/lowcode-utils';
import TreeNodeView from './tree-node';
import { IPublicEnumDragObjectType, IPublicTypeLocationDetailType } from '@gant-lowcode/lowcode-types';
import { BackupPaneName } from '../helper/consts';
function getTreeNodeIdByEvent(e, stop) {
  var target = e.target;
  if (!target || !stop.contains(target)) {
    return null;
  }
  target = target.closest('[data-id]');
  if (!target || !stop.contains(target)) {
    return null;
  }
  return target.dataset.id || null;
}
function findExpandedSlot(tree, dragObject, slots, start, step) {
  if (step === void 0) {
    step = 1;
  }
  for (var i = start; step > 0 ? i < slots.length : i >= 0; i += step) {
    var _tree$pluginContext$p;
    var slot = slots[i];
    var treeNode = tree.getTreeNodeById(slot.id);
    var valid = (_tree$pluginContext$p = tree.pluginContext.project.currentDocument) === null || _tree$pluginContext$p === void 0 ? void 0 : _tree$pluginContext$p.checkNesting(slot, dragObject);
    if ((treeNode !== null && treeNode !== void 0 && treeNode.expanded || !slot.children || slot.children.size === 0) && valid) {
      return slot;
    }
  }
  return;
}
function calculateTargetPosition(tree, dragObject, parent, sibling, direction) {
  var index = -1;
  var closest;
  if (sibling) {
    var _tree$pluginContext$p2;
    var siblingTreeNode = tree.getTreeNodeById(sibling.id);
    // 处理兄弟节点是容器的情况
    if (sibling.isContainerNode && (siblingTreeNode !== null && siblingTreeNode !== void 0 && siblingTreeNode.expanded || !sibling.children || sibling.children.size === 0) && (_tree$pluginContext$p2 = tree.pluginContext.project.currentDocument) !== null && _tree$pluginContext$p2 !== void 0 && _tree$pluginContext$p2.checkNesting(sibling, dragObject)) {
      var _sibling$children;
      closest = sibling;
      index = direction > 0 ? 0 : ((_sibling$children = sibling.children) === null || _sibling$children === void 0 ? void 0 : _sibling$children.size) || 0;
    }
    // 处理兄弟节点是插槽的情况
    if (!closest && sibling.hasSlots() && siblingTreeNode !== null && siblingTreeNode !== void 0 && siblingTreeNode.expanded) {
      var _closest, _closest$children;
      var slotIndex = direction > 0 ? 0 : sibling.slots.length - 1;
      closest = findExpandedSlot(tree, dragObject, sibling.slots, slotIndex, direction);
      index = direction > 0 ? 0 : ((_closest = closest) === null || _closest === void 0 ? void 0 : (_closest$children = _closest.children) === null || _closest$children === void 0 ? void 0 : _closest$children.size) || 0;
    }
    // 处理父容器是插槽的情况
    if (!closest && parent !== null && parent !== void 0 && parent.isSlotNode) {
      closest = parent;
      index = parent.children.indexOf(sibling) + (direction > 0 ? 1 : 0);
    }
    // 默认处理
    if (!closest && parent) {
      closest = parent;
      index = parent.children.indexOf(sibling) + (direction > 0 ? 1 : 0);
    }
  } else {
    var _tree$pluginContext$p3;
    var grandParent = parent === null || parent === void 0 ? void 0 : parent.parent;
    // 处理父节点是插槽的情况
    if (grandParent && parent !== null && parent !== void 0 && parent.isSlotNode) {
      var _closest2, _closest2$children;
      var parentIndex = grandParent.slots.findIndex(function (slot) {
        return slot.id === parent.id;
      }) || 0;
      var _slotIndex = direction > 0 ? parentIndex + 1 : parentIndex - 1;
      closest = findExpandedSlot(tree, dragObject, grandParent.slots, _slotIndex, direction);
      index = direction > 0 ? 0 : ((_closest2 = closest) === null || _closest2 === void 0 ? void 0 : (_closest2$children = _closest2.children) === null || _closest2$children === void 0 ? void 0 : _closest2$children.size) || 0;
    }
    // 处理祖父节点是容器的情况
    if (!closest && grandParent && grandParent.isContainerNode && (_tree$pluginContext$p3 = tree.pluginContext.project.currentDocument) !== null && _tree$pluginContext$p3 !== void 0 && _tree$pluginContext$p3.checkNesting(grandParent, dragObject)) {
      closest = grandParent;
      index = grandParent.children.indexOf(parent) + (direction > 0 ? 1 : 0);
    }
    // 递归处理
    if (!closest && grandParent) {
      return calculateTargetPosition(tree, dragObject, grandParent, undefined, direction);
    }
  }
  if (closest) {
    return [closest, index];
  }
}
var TreeView = function TreeView(_ref) {
  var tree = _ref.tree;
  var _useState = useState(tree.root),
    root = _useState[0],
    setRoot = _useState[1];
  var shellRef = useRef(null);
  var ignoreUpSelectedRef = useRef(false);
  var boostEventRef = useRef();
  var hover = function hover(e) {
    var _project$currentDocum, _getTreeNodeFromEvent;
    var project = tree.pluginContext.project;
    var detecting = (_project$currentDocum = project.currentDocument) === null || _project$currentDocum === void 0 ? void 0 : _project$currentDocum.detecting;
    var node = (_getTreeNodeFromEvent = getTreeNodeFromEvent(e)) === null || _getTreeNodeFromEvent === void 0 ? void 0 : _getTreeNodeFromEvent.node;
    (node === null || node === void 0 ? void 0 : node.id) && (detecting === null || detecting === void 0 ? void 0 : detecting.capture(node.id));
  };
  var getTreeNodeFromEvent = function getTreeNodeFromEvent(e) {
    if (!shellRef.current) return;
    var id = getTreeNodeIdByEvent(e, shellRef.current);
    return id ? tree.getTreeNodeById(id) : undefined;
  };
  var handleClick = function handleClick(e) {
    var _canvas$activeTracker;
    if (ignoreUpSelectedRef.current) {
      boostEventRef.current = undefined;
      return;
    }
    if (boostEventRef.current && isShaken(boostEventRef.current, e.nativeEvent)) {
      boostEventRef.current = undefined;
      return;
    }
    boostEventRef.current = undefined;
    var treeNode = getTreeNodeFromEvent(e);
    if (!treeNode) return;
    var node = treeNode.node;
    if (!canClickNode(node, e)) return;
    var _tree$pluginContext = tree.pluginContext,
      project = _tree$pluginContext.project,
      event = _tree$pluginContext.event,
      canvas = _tree$pluginContext.canvas;
    var doc = project.currentDocument;
    var selection = doc === null || doc === void 0 ? void 0 : doc.selection;
    var focusNode = doc === null || doc === void 0 ? void 0 : doc.focusNode;
    var id = node.id;
    var isMulti = e.metaKey || e.ctrlKey || e.shiftKey;
    (_canvas$activeTracker = canvas.activeTracker) === null || _canvas$activeTracker === void 0 ? void 0 : _canvas$activeTracker.track(node);
    if (selection !== null && selection !== void 0 && selection.hasAnchord) return;
    if (isMulti && focusNode && !node.contains(focusNode) && selection !== null && selection !== void 0 && selection.has(id)) {
      if (!isFormEvent(e.nativeEvent)) {
        selection.remove(id);
      }
    } else {
      var _selection$getNodes, _selectedNode$compone, _selectedNode$compone2;
      selection === null || selection === void 0 ? void 0 : selection.select(id);
      var selectedNode = selection === null || selection === void 0 ? void 0 : (_selection$getNodes = selection.getNodes()) === null || _selection$getNodes === void 0 ? void 0 : _selection$getNodes[0];
      var npm = selectedNode === null || selectedNode === void 0 ? void 0 : (_selectedNode$compone = selectedNode.componentMeta) === null || _selectedNode$compone === void 0 ? void 0 : _selectedNode$compone.npm;
      var selected = [npm === null || npm === void 0 ? void 0 : npm["package"], npm === null || npm === void 0 ? void 0 : npm.componentName].filter(Boolean).join('-') || (selectedNode === null || selectedNode === void 0 ? void 0 : (_selectedNode$compone2 = selectedNode.componentMeta) === null || _selectedNode$compone2 === void 0 ? void 0 : _selectedNode$compone2.componentName) || '';
      event.emit('outlinePane.select', {
        selected: selected
      });
    }
  };
  var handleDoubleClick = function handleDoubleClick(e) {
    e.preventDefault();
    var treeNode = getTreeNodeFromEvent(e);
    if ((treeNode === null || treeNode === void 0 ? void 0 : treeNode.nodeId) === (root === null || root === void 0 ? void 0 : root.nodeId)) return;
    if (!(treeNode !== null && treeNode !== void 0 && treeNode.expanded)) {
      tree.expandAllDecendants(treeNode);
    } else {
      tree.collapseAllDecendants(treeNode);
    }
  };
  var handleMouseDown = function handleMouseDown(e) {
    var _project$currentDocum2, _project$currentDocum3;
    if (isFormEvent(e.nativeEvent)) return;
    var treeNode = getTreeNodeFromEvent(e);
    if (!treeNode) return;
    var node = treeNode.node;
    if (!canClickNode(node, e)) return;
    var _tree$pluginContext2 = tree.pluginContext,
      project = _tree$pluginContext2.project,
      canvas = _tree$pluginContext2.canvas;
    var selection = (_project$currentDocum2 = project.currentDocument) === null || _project$currentDocum2 === void 0 ? void 0 : _project$currentDocum2.selection;
    var focusNode = (_project$currentDocum3 = project.currentDocument) === null || _project$currentDocum3 === void 0 ? void 0 : _project$currentDocum3.focusNode;
    var isMulti = e.metaKey || e.ctrlKey || e.shiftKey;
    var isLeftButton = e.button === 0;
    if (isLeftButton && focusNode && !node.contains(focusNode)) {
      var _canvas$dragon;
      var nodes = [node];
      ignoreUpSelectedRef.current = false;
      if (isMulti) {
        // multi select mode, directily add
        if (!(selection !== null && selection !== void 0 && selection.has(node.id))) {
          var _canvas$activeTracker2;
          (_canvas$activeTracker2 = canvas.activeTracker) === null || _canvas$activeTracker2 === void 0 ? void 0 : _canvas$activeTracker2.track(node);
          selection === null || selection === void 0 ? void 0 : selection.add(node.id);
          ignoreUpSelectedRef.current = true;
        }
        // todo: remove rootNodes id
        selection === null || selection === void 0 ? void 0 : selection.remove(focusNode.id);
        // 获得顶层 nodes
        if (selection) {
          nodes = selection.getTopNodes();
        }
      } else if (selection !== null && selection !== void 0 && selection.has(node.id)) {
        nodes = selection.getTopNodes();
      }
      boostEventRef.current = e.nativeEvent;
      (_canvas$dragon = canvas.dragon) === null || _canvas$dragon === void 0 ? void 0 : _canvas$dragon.boost({
        type: IPublicEnumDragObjectType.Node,
        nodes: nodes,
        from: 'outline'
      }, boostEventRef.current);
    }
  };
  var handleMouseLeave = function handleMouseLeave() {
    var _project$currentDocum4;
    var project = tree.pluginContext.project;
    (_project$currentDocum4 = project.currentDocument) === null || _project$currentDocum4 === void 0 ? void 0 : _project$currentDocum4.detecting.leave();
  };
  var handleKeyDown = function handleKeyDown(e) {
    var _skeleton$getPanel, _doc$selection, _nodes$, _canvas$dragon2, _canvas$dragon3;
    var _tree$pluginContext3 = tree.pluginContext,
      project = _tree$pluginContext3.project,
      canvas = _tree$pluginContext3.canvas,
      skeleton = _tree$pluginContext3.skeleton;
    if (!((_skeleton$getPanel = skeleton.getPanel(BackupPaneName)) !== null && _skeleton$getPanel !== void 0 && _skeleton$getPanel.visible)) return;
    var doc = project.getCurrentDocument();
    var nodes = (doc === null || doc === void 0 ? void 0 : (_doc$selection = doc.selection) === null || _doc$selection === void 0 ? void 0 : _doc$selection.getNodes()) || [];
    var parent = (_nodes$ = nodes[0]) === null || _nodes$ === void 0 ? void 0 : _nodes$.parent;
    // 验证选中的节点是否属于同一个父节点
    if (nodes.length === 0 || !parent || !nodes.every(function (n) {
      var _n$parent;
      return ((_n$parent = n.parent) === null || _n$parent === void 0 ? void 0 : _n$parent.id) === (parent === null || parent === void 0 ? void 0 : parent.id);
    })) return;
    var currentNodes = parent.children;
    var indexes = nodes.map(function (n) {
      return currentNodes.indexOf(n);
    }).filter(function (i) {
      return i > -1;
    });
    var firstIndex = Math.min.apply(Math, indexes);
    var lastIndex = Math.max.apply(Math, indexes);
    // 确定移动方向
    var direction = ['ArrowUp', 'ArrowLeft'].includes(e.key) ? -1 : 1;
    var node = direction > 0 ? currentNodes.get(lastIndex) : currentNodes.get(firstIndex);
    var nodeParent = node === null || node === void 0 ? void 0 : node.parent;
    if (!node || !nodeParent) return;
    var dragObject = {
      type: IPublicEnumDragObjectType.Node,
      nodes: nodes
    };
    console.log('---dragObject', dragObject);
    var targetData;
    // 根据按键类型调用不同的处理函数
    switch (e.key) {
      case 'ArrowUp':
        targetData = calculateTargetPosition(tree, dragObject, node.parent, node.prevSibling, -1);
        break;
      case 'ArrowDown':
        targetData = calculateTargetPosition(tree, dragObject, node.parent, node.nextSibling, 1);
        break;
      case 'ArrowLeft':
        targetData = calculateTargetPosition(tree, dragObject, node.parent, undefined, 1);
        break;
      case 'ArrowRight':
        targetData = calculateTargetPosition(tree, dragObject, null, node.prevSibling, -1);
        break;
      default:
        return;
    }
    console.log('---targetData', targetData);
    if (!targetData) return;
    var _targetData = targetData,
      container = _targetData[0],
      targetIndex = _targetData[1];
    var evt = {
      type: 'LocateEvent',
      dragObject: dragObject,
      target: document.querySelector(".tree-node[data-id=\"" + node.id + "\"]"),
      originalEvent: e
    };
    canvas.createLocation({
      target: container,
      event: evt,
      source: 'keyword',
      detail: {
        type: IPublicTypeLocationDetailType.Children,
        index: targetIndex,
        valid: true
      }
    });
    (_canvas$dragon2 = canvas.dragon) === null || _canvas$dragon2 === void 0 ? void 0 : _canvas$dragon2.emitDragEnd(dragObject);
    (_canvas$dragon3 = canvas.dragon) === null || _canvas$dragon3 === void 0 ? void 0 : _canvas$dragon3.clearLocation();
  };
  useEffect(function () {
    var _tree$pluginContext4 = tree.pluginContext,
      project = _tree$pluginContext4.project,
      hotkey = _tree$pluginContext4.hotkey;
    var doc = project.currentDocument;
    var handleFocusChange = function handleFocusChange() {
      return setRoot(tree.root);
    };
    var handleImportSchema = function handleImportSchema() {
      return setRoot(tree.root);
    };
    var disposables = [doc === null || doc === void 0 ? void 0 : doc.onFocusNodeChanged(handleFocusChange), doc === null || doc === void 0 ? void 0 : doc.onImportSchema(handleImportSchema), hotkey.bind(['shift+up', 'shift+down', 'shift+left', 'shift+right'], handleKeyDown)];
    return function () {
      disposables.forEach(function (d) {
        return d === null || d === void 0 ? void 0 : d();
      });
    };
  }, [tree, handleKeyDown]);
  if (!root) return null;
  return /*#__PURE__*/React.createElement("div", {
    className: "lc-outline-tree",
    ref: shellRef,
    onMouseDownCapture: handleMouseDown,
    onMouseOver: hover,
    onClick: handleClick,
    onDoubleClick: handleDoubleClick,
    onMouseLeave: handleMouseLeave
  }, /*#__PURE__*/React.createElement(TreeNodeView, {
    key: root.id,
    treeNode: root,
    isRootNode: true
  }));
};
export default TreeView;