"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.TreeMaster = void 0;
var _button = _interopRequireDefault(require("antd/lib/button"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _lowcodeUtils = require("@gant-lowcode/lowcode-utils");
var _lowcodeTypes = require("@gant-lowcode/lowcode-types");
var _tree = require("./tree");
var _events = _interopRequireDefault(require("events"));
var _locale = require("../locale");
var _react = _interopRequireDefault(require("react"));
var EVENT_NAMES = /*#__PURE__*/function (EVENT_NAMES) {
  EVENT_NAMES["pluginContextChanged"] = "pluginContextChanged";
  return EVENT_NAMES;
}(EVENT_NAMES || {});
var setDontShowAgain = function setDontShowAgain() {
  return localStorage.setItem('dontShowOutlineShortcutPrompt', 'true');
};
var shouldShowPrompt = function shouldShowPrompt() {
  return localStorage.getItem('dontShowOutlineShortcutPrompt') !== 'true';
};

/**
 * 大纲树核心控制器，负责：
 * 1. 管理多个文档对应的大纲树实例
 * 2. 协调面板间的交互通信
 * 3. 处理全局事件与文档关联
 */
var TreeMaster = exports.TreeMaster = /*#__PURE__*/function () {
  function TreeMaster(pluginContext, options, notificationApi) {
    var _this = this;
    this.options = options;
    // 插件上下文，包含低代码引擎的运行时环境
    this.pluginContext = void 0;
    // 注册的面板实例集合（观察者模式）
    this.boards = new Set();
    // 文档ID与大纲树的映射表
    this.treeMap = new Map();
    // 事件销毁器集合
    this.disposeEvents = [];
    this.notificationApi = void 0;
    this.event = new _events["default"]();
    this.dragCount = 0;
    this.setPluginContext(pluginContext);
    this.notificationApi = notificationApi;
    var workspace = this.pluginContext.workspace;
    this.initEvent();
    // 工作区级别的插件注册逻辑
    if (pluginContext.registerLevel === _lowcodeTypes.IPublicEnumPluginRegisterLevel.Workspace) {
      var _workspace$window;
      this.setPluginContext((_workspace$window = workspace.window) === null || _workspace$window === void 0 ? void 0 : _workspace$window.currentEditorView);
      var dispose;
      var windowViewTypeChangeEvent = function windowViewTypeChangeEvent() {
        var _workspace$window2;
        dispose = (_workspace$window2 = workspace.window) === null || _workspace$window2 === void 0 ? void 0 : _workspace$window2.onChangeViewType(function () {
          var _workspace$window3;
          _this.setPluginContext((_workspace$window3 = workspace.window) === null || _workspace$window3 === void 0 ? void 0 : _workspace$window3.currentEditorView);
        });
      };
      windowViewTypeChangeEvent();
      workspace.onChangeActiveWindow(function () {
        var _workspace$window4;
        _this.setPluginContext((_workspace$window4 = workspace.window) === null || _workspace$window4 === void 0 ? void 0 : _workspace$window4.currentEditorView);
        dispose && dispose();
        windowViewTypeChangeEvent();
      });
    }
  }
  var _proto = TreeMaster.prototype;
  _proto.setNotificationApi = function setNotificationApi(api) {
    this.notificationApi = api;
  };
  _proto.setPluginContext = function setPluginContext(pluginContext) {
    if (!pluginContext) {
      return;
    }
    var _pluginContext$common = pluginContext.common.utils.createIntl({
        'en-US': _locale.enUS,
        'zh-CN': _locale.zhCN
      }),
      intl = _pluginContext$common.intl,
      intlNode = _pluginContext$common.intlNode,
      getLocale = _pluginContext$common.getLocale;
    var _pluginContext = Object.assign(pluginContext, {
      intl: intl,
      intlNode: intlNode,
      getLocale: getLocale
    });
    _pluginContext.extraTitle = this.options && this.options['extraTitle'];
    this.pluginContext = _pluginContext;
    this.disposeEvent();
    this.initEvent();
    this.emitPluginContextChange();
  };
  _proto.disposeEvent = function disposeEvent() {
    this.disposeEvents.forEach(function (d) {
      d && d();
    });
  };
  /**
   * 初始化全局事件监听
   * 包含：
   * - 拖拽开始/结束事件
   * - 激活节点追踪器变更事件
   * - 文档移除事件
   */
  _proto.initEvent = function initEvent() {
    var _this2 = this,
      _canvas$dragon,
      _canvas$activeTracker,
      _canvas$dragon2,
      _canvas$activeTracker2;
    var startTime;
    var _this$pluginContext = this.pluginContext,
      event = _this$pluginContext.event,
      project = _this$pluginContext.project,
      canvas = _this$pluginContext.canvas;
    var setExpandByActiveTracker = function setExpandByActiveTracker(target) {
      var node = target.node,
        detail = target.detail;
      var tree = _this2.currentTree;
      if (!tree /* || node.document !== tree.document */) {
        return;
      }
      var treeNode = tree.getTreeNode(node);
      // 根据位置详情决定展开方式
      if (detail && (0, _lowcodeUtils.isLocationChildrenDetail)(detail)) {
        treeNode.expand(true); // 直接展开目标节点
      } else {
        treeNode.expandParents(); // 展开父级链
      }

      // 通知所有关联面板滚动到目标节点
      _this2.boards.forEach(function (board) {
        board.scrollToNode(treeNode, detail);
      });
    };
    this.disposeEvents = [(_canvas$dragon = canvas.dragon) === null || _canvas$dragon === void 0 ? void 0 : _canvas$dragon.onDragstart(function () {
      startTime = Date.now() / 1000;
      // needs?
      _this2.toVision();
    }), (_canvas$activeTracker = canvas.activeTracker) === null || _canvas$activeTracker === void 0 ? void 0 : _canvas$activeTracker.onChange(setExpandByActiveTracker), (_canvas$dragon2 = canvas.dragon) === null || _canvas$dragon2 === void 0 ? void 0 : _canvas$dragon2.onDragend(function (_ref) {
      var _project$currentDocum, _project$currentDocum2;
      var dragObject = _ref.dragObject;
      console.log('--结束drag');
      var endTime = Date.now() / 1000;
      var nodes = (_project$currentDocum = project.currentDocument) === null || _project$currentDocum === void 0 ? void 0 : (_project$currentDocum2 = _project$currentDocum.selection) === null || _project$currentDocum2 === void 0 ? void 0 : _project$currentDocum2.getNodes();
      if (dragObject.from === 'outline') {
        if (++_this2.dragCount >= 3 && shouldShowPrompt()) {
          var _this2$notificationAp2;
          var btn = /*#__PURE__*/_react["default"].createElement(_button["default"], {
            type: 'link',
            size: 'small',
            onClick: function onClick() {
              var _this2$notificationAp;
              setDontShowAgain();
              (_this2$notificationAp = _this2.notificationApi) === null || _this2$notificationAp === void 0 ? void 0 : _this2$notificationAp.destroy();
            }
          }, '不再提示');
          (_this2$notificationAp2 = _this2.notificationApi) === null || _this2$notificationAp2 === void 0 ? void 0 : _this2$notificationAp2.open({
            key: 'outline-drag',
            message: '快捷键提示',
            description: '可以在按下 shift 键的同时按下上下键，将选中组件移动到其他位置。',
            btn: btn,
            placement: 'bottomRight'
          });
        }
      }
      event.emit('outlinePane.dragend', {
        selected: nodes === null || nodes === void 0 ? void 0 : nodes.map(function (n) {
          var _n$componentMeta, _n$componentMeta2;
          if (!n) {
            return;
          }
          var npm = n === null || n === void 0 ? void 0 : (_n$componentMeta = n.componentMeta) === null || _n$componentMeta === void 0 ? void 0 : _n$componentMeta.npm;
          return [npm === null || npm === void 0 ? void 0 : npm["package"], npm === null || npm === void 0 ? void 0 : npm.componentName].filter(function (item) {
            return !!item;
          }).join('-') || (n === null || n === void 0 ? void 0 : (_n$componentMeta2 = n.componentMeta) === null || _n$componentMeta2 === void 0 ? void 0 : _n$componentMeta2.componentName);
        }).join('&'),
        time: (endTime - startTime).toFixed(2)
      });
    }), project.onRemoveDocument(function (data) {
      var id = data.id;
      _this2.treeMap["delete"](id);
    })];
    if ((_canvas$activeTracker2 = canvas.activeTracker) !== null && _canvas$activeTracker2 !== void 0 && _canvas$activeTracker2.target) {
      var _canvas$activeTracker3;
      setExpandByActiveTracker((_canvas$activeTracker3 = canvas.activeTracker) === null || _canvas$activeTracker3 === void 0 ? void 0 : _canvas$activeTracker3.target);
    }
  };
  _proto.toVision = function toVision() {
    var tree = this.currentTree;
    if (tree) {
      var _this$pluginContext$p;
      var selection = (_this$pluginContext$p = this.pluginContext.project.getCurrentDocument()) === null || _this$pluginContext$p === void 0 ? void 0 : _this$pluginContext$p.selection;
      selection === null || selection === void 0 ? void 0 : selection.getTopNodes().forEach(function (node) {
        tree.getTreeNode(node).setExpanded(false);
      });
    }
  };
  _proto.addBoard = function addBoard(board) {
    this.boards.add(board);
  };
  _proto.removeBoard = function removeBoard(board) {
    this.boards["delete"](board);
  };
  _proto.purge = function purge() {
    // todo others purge
  };
  _proto.onPluginContextChange = function onPluginContextChange(fn) {
    this.event.on(EVENT_NAMES.pluginContextChanged, fn);
  };
  _proto.emitPluginContextChange = function emitPluginContextChange() {
    this.event.emit(EVENT_NAMES.pluginContextChanged);
  }

  /**
   * 获取当前文档对应的大纲树
   * 每个文档维护独立的大纲树实例
   */;
  return (0, _createClass2["default"])(TreeMaster, [{
    key: "currentTree",
    get: function get() {
      var doc = this.pluginContext.project.getCurrentDocument();
      if (doc) {
        var _id = doc.id;
        if (this.treeMap.has(_id)) {
          return this.treeMap.get(_id);
        }
        var tree = new _tree.Tree(this);
        this.treeMap.set(_id, tree);
        return tree;
      }
      return null;
    }
  }]);
}();